/*-----------------------------------------------------------------
	
	Compute the space of quartic relations on a Kummer 

        variety associated to a given hyperelliptic genus 3

        curve with a rational Weierstrass point at infinity.

        Together with check_inpendence.m, this file verifies

        Theorem 3.3 of the paper Explicit Kummer varieties of

        hyperelliptic Jacobian threefolds.
        

	-- Steffen Mueller --

-----------------------------------------------------------------*/

K<f0,f1,f2,f3,f4,f5,f6,f7> := FunctionField(Rationals(), 8);
f_coeffs := [f0,f1,f2,f3,f4,f5,f6,f7];
P6<x1, x2, x3, y1, y2, y3> := PolynomialRing(K, 6);
F6 := FieldOfFractions(P6);

function quartic_monomials(a, c)
    // Compute all quartic monomials in the a_i and c_i.
    X := [];
    for i := 1 to 4 do
        for j := i to 4 do
            for k := j to 4 do 
                for l := k to 4 do
                    Append(~X, F6!(a[i] * a[j] * a[k] * a[l]));
                    Append(~X, c[i] * c[j] * c[k] * c[l]);
                end for;
            end for;
        end for;
    end for;
    for i := 1 to 4 do
        for j := i to 4 do
            for k := j to 4 do 
                for l := 1 to 4 do
                    Append(~X, a[i] * a[j] * a[k] * c[l]);
                    Append(~X, c[i] * c[j] * c[k] * a[l]);
                end for;
            end for;
        end for;
    end for;
    for i := 1 to 4 do
        for j := i to 4 do
            for k := 1 to 4 do 
                for l := k to 4 do
                    Append(~X, a[i] * a[j] * c[k] * c[l]);
                end for;
            end for;
        end for;
    end for;
    return X;
end function;



function subsy2(a0, F)
// This function substitutes y_i^2 by f(x_i) in a0.
// Here a0 is an element of P6.
    a := a0;
    // Substitute y_1.
    cc1 := Coefficients(a, 4);
    for i := 2 to #cc1 - 1 do
        j := i div 2;
        r := i mod 2; 
        a1 := a - y1^i * cc1[i + 1]; 
        a := a1 + F(x1)^j * y1^r * cc1[i + 1];
    end for;
    // Substitute y_2.
    cc2 := Coefficients(a, 5);
    for i := 2 to #cc2 - 1 do
        j := i div 2;
        r := i mod 2; 
        a1 := a - y2^i * cc2[i + 1]; 
        a := a1 + F(x2)^j * y2^r * cc2[i + 1];
    end for;
    // Substitute y_3.
    cc3 := Coefficients(a, 6);
    for i := 2 to #cc3 - 1 do
        j := i div 2;
        r := i mod 2; 
        a1 := a - y3^i * cc3[i + 1]; 
        a := a1 + F(x3)^j * y3^r * cc3[i + 1];
    end for;
    return a;
end function;

function comp_kernel(T, F)
// Compute a basis for the space of linear relations between elements of T.
// Here T is a set of quartic monomials in the a_i, c_i.

// First create a sequence containing all y_1*y_2-coefficients of the numerators of 
// elements of S. These are polynomials in the x_i.
// It suffices to find relations between the y_1*y_2-coefficients as in Chapter 5
// of Stubbs's thesis.
    x1, x2, x3 := Explode([Universe(T).i : i  in [1..3]]); 
    delta := (x1 - x2) * (x2 - x3) * (x3 - x1);
    S := [];
    for i := 1 to #T do
    // Compute the coefficient of y_1*y_2 in the numerator of T[i].
        s := Coefficient(Coefficient(subsy2(P6 ! (F6 ! delta^8 * T[i]), F), 4, 1), 5, 1);
        if s ne 0 then
            Append(~S, s);
        end if;
    end for; 
    // Create a sequence containing all monomials in the x_i appearing in S.
    L := [];
    for i := 1 to #S do
        mi := Monomials(S[i]);
        for j := 1 to #mi do
            Include(~L, mi[j]); 
        end for;
    end for;
    // Create a sequence containing all monomial coefficients of elements of S.
    U := [];
    for i := 1 to #S do 
        Z := [];
        for j := 1 to #L do  
            Append(~Z, MonomialCoefficient(S[i], L[j]));
        end for;
        Append(~U, Z);
    end for;
    // Now compute a basis of the space of relations satisfied by elements of U.
    K := KernelMatrix(Matrix(U));
    return [Eltseq(K[i]):i in [1..Nrows(K)]];
end function;


function quartic_relations(f_coefficients)
// Compute the space of quartic relations on the Kummer variety 
// of the Jacobian of the curve y^2 = f(x), where f_coefficients are the 
// coefficients of f.

    f0, f1, f2, f3, f4, f5, f6, f7 := Explode(f_coefficients);

    // We assume the curve is given as y^2 = f(x)

    function F(x)
        return f0 + f1 * x + f2 * x^2 + f3 * x^3 + f4 * x^4 + f5 * x^5 + f6 * x^6 + f7 * x^7;
    end function;

    // We employ the notation used by Stubbs in his Liverpool PhD thesis.
    a0 := 1;
    a1 := x1 + x2 + x3;
    a2 := x1 * x2 + x2 * x3 + x1 * x3;
    a3 := x1 * x2 * x3;
    delta := (x1 - x2) * (x2 - x3) * (x3 - x1);
    H0 :=  - f7 * a1^3 + f7 * a2 * a1 - f6 * a1^2 + 3 * f7 * a3 + 2 * f6 * a2;
    H1 :=  - f7 * a1^4 + 3 * f7 * a2 * a1^2 - f6 * a1^3 - f7 * a2^2 - f7 * a3 * a1 + 2 * f6 *
    a2 * a1 - f5 * a1^2 + 2 * f5 * a2;
    H2 := f7 * a2 * a1^3 - 2 * f7 * a2^2 * a1 + f6 * a2 * a1^2 + f7 * a3 * a2 - f6 * a2^2 + f5
    * a2 * a1 - 3 * f5 * a3;
    H3 := f7 * a2^2 * a1^2 - f7 * a1^3 * a3 + f7 * a1 * a2 * a3 - f7 * a2^3 + f6 * a2^2 * a1 -
    f6 * a3 * a1^2 + f5 * a2^2 - f5 * a3 * a1;
    b0 := (x1 * y2 - x2 * y1 - x3 * y2 + x3 * y1 - x1 * y3 + x2 * y3) / delta;
    b1 := (x3^2 * y2 - x3^2 * y1 + x2^2 * y1 + y3 * x1^2 - y2 * x1^2 - y3 * x2^2) / delta;

    // Use the non - weight - reduced coordinates. 
    // In Stubbs' thesis these are gamma_0,...,gamma3.
    C0 := a0 * b0^2;
    C1 := a1 * b0^2 + 2 * a0 * b0 * b1;
    C2 := a0 * b1^2 - a2 * b0^2;
    C3 := a1 * b1^2 + 2 * a2 * b0 * b1 + a3 * b0^2;
    c0 := C0 + H0;
    c1 := C1 + H1;
    c2 := C2 + H2;
    c3 := C3 + H3;
    a := [a0, a1, a2, a3];
    c := [c0, c1, c2, c3];
    
    // Compute all quartic monomials in the a_i and c_i
    X := quartic_monomials(a, c);
    // Compute the relations
    L := comp_kernel(X, F);
    
    return L, X; 
end function;


// Check that there are 70 independent quartic relations on the Kummer variety
// of the curve  given by y^2 = x^7 + 1.

#quartic_relations([1,0,0,0,0,0,0,1]);

